@@ -23,23 +23,14 @@
 import os.path
 
 try:
-    from ._version import __version__ as version  # type: ignore
+    from bpython._version import __version__ as version
 except ImportError:
-    version = "unknown"
+    version = 'unkown'
 
-__author__ = (
-    "Bob Farrell, Andreas Stuehrk, Sebastian Ramacher, Thomas Ballinger, et al."
-)
-__copyright__ = f"(C) 2008-2020 {__author__}"
-__license__ = "MIT"
 __version__ = version
 package_dir = os.path.abspath(os.path.dirname(__file__))
 
 
-def embed(locals_=None, args=None, banner=None):
-    if args is None:
-        args = ["-i", "-q"]
-
-    from .curtsies import main
-
+def embed(locals_=None, args=['-i', '-q'], banner=None):
+    from bpython.curtsies import main
     return main(args, locals_, banner)
