@@ -1,6 +1,9 @@
+# encoding: utf-8
+
 from collections import namedtuple
 import inspect
-from bpython._py3compat import py3
+import keyword
+import sys
 
 try:
     import unittest2 as unittest
@@ -14,13 +17,20 @@ except ImportError:
     has_jedi = False
 
 from bpython import autocomplete
+from bpython._py3compat import py3
 from bpython.test import mock
 
+is_py34 = sys.version_info[:2] >= (3, 4)
+if is_py34:
+    glob_function = 'glob.iglob'
+else:
+    glob_function = 'glob.glob'
+
 
 class TestSafeEval(unittest.TestCase):
     def test_catches_syntax_error(self):
-        self.assertRaises(autocomplete.EvaluationError,
-                          autocomplete.safe_eval, '1re', {})
+        with self.assertRaises(autocomplete.EvaluationError):
+            autocomplete.safe_eval('1re', {})
 
 
 class TestFormatters(unittest.TestCase):
@@ -97,10 +107,10 @@ class TestCumulativeCompleter(unittest.TestCase):
         cumulative = autocomplete.CumulativeCompleter([a])
         self.assertEqual(cumulative.matches(3, 'abc'), set())
 
-    def test_one_none_completer_returns_empty(self):
+    def test_one_none_completer_returns_none(self):
         a = self.completer(None)
         cumulative = autocomplete.CumulativeCompleter([a])
-        self.assertEqual(cumulative.matches(3, 'abc'), set())
+        self.assertEqual(cumulative.matches(3, 'abc'), None)
 
     def test_two_completers_get_both(self):
         a = self.completer(['a'])
@@ -120,16 +130,18 @@ class TestFilenameCompletion(unittest.TestCase):
     def test_locate_succeeds_when_in_string(self):
         self.assertEqual(self.completer.locate(4, "a'bc'd"), (2, 4, 'bc'))
 
-    @mock.patch('glob.glob', new=lambda text: [])
+    def test_issue_491(self):
+        self.assertNotEqual(self.completer.matches(9, '"a[a.l-1]'), None)
+
+    @mock.patch(glob_function, new=lambda text: [])
     def test_match_returns_none_if_not_in_string(self):
         self.assertEqual(self.completer.matches(2, 'abcd'), None)
 
-    @mock.patch('glob.glob', new=lambda text: [])
+    @mock.patch(glob_function, new=lambda text: [])
     def test_match_returns_empty_list_when_no_files(self):
         self.assertEqual(self.completer.matches(2, '"a'), set())
 
-    @mock.patch('glob.glob',
-                new=lambda text: ['abcde', 'aaaaa'])
+    @mock.patch(glob_function, new=lambda text: ['abcde', 'aaaaa'])
     @mock.patch('os.path.expanduser', new=lambda text: text)
     @mock.patch('os.path.isdir', new=lambda text: False)
     @mock.patch('os.path.sep', new='/')
@@ -137,8 +149,7 @@ class TestFilenameCompletion(unittest.TestCase):
         self.assertEqual(sorted(self.completer.matches(2, '"x')),
                          ['aaaaa', 'abcde'])
 
-    @mock.patch('glob.glob',
-                new=lambda text: ['abcde', 'aaaaa'])
+    @mock.patch(glob_function, new=lambda text: ['abcde', 'aaaaa'])
     @mock.patch('os.path.expanduser', new=lambda text: text)
     @mock.patch('os.path.isdir', new=lambda text: True)
     @mock.patch('os.path.sep', new='/')
@@ -146,7 +157,7 @@ class TestFilenameCompletion(unittest.TestCase):
         self.assertEqual(sorted(self.completer.matches(2, '"x')),
                          ['aaaaa/', 'abcde/'])
 
-    @mock.patch('glob.glob',
+    @mock.patch(glob_function,
                 new=lambda text: ['/expand/ed/abcde', '/expand/ed/aaaaa'])
     @mock.patch('os.path.expanduser',
                 new=lambda text: text.replace('~', '/expand/ed'))
@@ -179,20 +190,25 @@ class TestDictKeyCompletion(unittest.TestCase):
         self.assertSetEqual(com.matches(2, "d[", locals_=local),
                             set(["'ab']", "'cd']"]))
 
-    def test_empty_set_returned_when_eval_error(self):
+    def test_none_returned_when_eval_error(self):
         com = autocomplete.DictKeyCompletion()
         local = {'e': {"ab": 1, "cd": 2}}
-        self.assertSetEqual(com.matches(2, "d[", locals_=local), set())
+        self.assertEqual(com.matches(2, "d[", locals_=local), None)
 
-    def test_empty_set_returned_when_not_dict_type(self):
+    def test_none_returned_when_not_dict_type(self):
         com = autocomplete.DictKeyCompletion()
         local = {'l': ["ab", "cd"]}
-        self.assertSetEqual(com.matches(2, "l[", locals_=local), set())
+        self.assertEqual(com.matches(2, "l[", locals_=local), None)
+
+    def test_none_returned_when_no_matches_left(self):
+        com = autocomplete.DictKeyCompletion()
+        local = {'d': {"ab": 1, "cd": 2}}
+        self.assertEqual(com.matches(3, "d[r", locals_=local), None)
 
     def test_obj_that_does_not_allow_conversion_to_bool(self):
         com = autocomplete.DictKeyCompletion()
         local = {'mNumPy': MockNumPy()}
-        self.assertSetEqual(com.matches(7, "mNumPy[", locals_=local), set())
+        self.assertEqual(com.matches(7, "mNumPy[", locals_=local), None)
 
 
 class Foo(object):
@@ -205,13 +221,129 @@ class Foo(object):
         pass
 
 
+class OldStyleFoo:
+    a = 10
+
+    def __init__(self):
+        self.b = 20
+
+    def method(self, x):
+        pass
+
+
+skip_old_style = unittest.skipIf(py3,
+                                 'In Python 3 there are no old style classes')
+
+
+class Properties(Foo):
+
+    @property
+    def asserts_when_called(self):
+        raise AssertionError("getter method called")
+
+
+class Slots(object):
+    __slots__ = ['a', 'b']
+
+
 class TestAttrCompletion(unittest.TestCase):
+    @classmethod
+    def setUpClass(cls):
+        cls.com = autocomplete.AttrCompletion()
 
     def test_att_matches_found_on_instance(self):
-        com = autocomplete.AttrCompletion()
-        self.assertSetEqual(com.matches(2, 'a.', locals_={'a': Foo()}),
+        self.assertSetEqual(self.com.matches(2, 'a.', locals_={'a': Foo()}),
                             set(['a.method', 'a.a', 'a.b']))
 
+    @skip_old_style
+    def test_att_matches_found_on_old_style_instance(self):
+        self.assertSetEqual(self.com.matches(2, 'a.',
+                                             locals_={'a': OldStyleFoo()}),
+                            set(['a.method', 'a.a', 'a.b']))
+        self.assertIn(u'a.__dict__',
+                      self.com.matches(4, 'a.__',
+                                       locals_={'a': OldStyleFoo()}))
+
+    @skip_old_style
+    def test_att_matches_found_on_old_style_class_object(self):
+        self.assertIn(u'A.__dict__',
+                      self.com.matches(4, 'A.__', locals_={'A': OldStyleFoo}))
+
+    @skip_old_style
+    def test_issue536(self):
+        class OldStyleWithBrokenGetAttr:
+            def __getattr__(self, attr):
+                raise Exception()
+
+        locals_ = {'a': OldStyleWithBrokenGetAttr()}
+        self.assertIn(u'a.__module__',
+                      self.com.matches(4, 'a.__', locals_=locals_))
+
+    def test_descriptor_attributes_not_run(self):
+        com = autocomplete.AttrCompletion()
+        self.assertSetEqual(com.matches(2, 'a.', locals_={'a': Properties()}),
+                            set(['a.b', 'a.a', 'a.method',
+                                 'a.asserts_when_called']))
+
+    def test_slots_not_crash(self):
+        com = autocomplete.AttrCompletion()
+        self.assertSetEqual(com.matches(2, 'A.', locals_={'A': Slots}),
+                            set(['A.b', 'A.a', 'A.mro']))
+
+
+class TestExpressionAttributeCompletion(unittest.TestCase):
+    @classmethod
+    def setUpClass(cls):
+        cls.com = autocomplete.ExpressionAttributeCompletion()
+
+    def test_att_matches_found_on_instance(self):
+        self.assertSetEqual(self.com.matches(5, 'a[0].',
+                                             locals_={'a': [Foo()]}),
+                            set(['method', 'a', 'b']))
+
+    @skip_old_style
+    def test_att_matches_found_on_old_style_instance(self):
+        self.assertSetEqual(self.com.matches(5, 'a[0].',
+                                             locals_={'a': [OldStyleFoo()]}),
+                            set(['method', 'a', 'b']))
+
+    def test_other_getitem_methods_not_called(self):
+        class FakeList(object):
+            def __getitem__(inner_self, i):
+                self.fail("possibly side-effecting __getitem_ method called")
+
+        self.com.matches(5, 'a[0].', locals_={'a': FakeList()})
+
+    def test_tuples_complete(self):
+        self.assertSetEqual(self.com.matches(5, 'a[0].',
+                            locals_={'a': (Foo(),)}),
+                            set(['method', 'a', 'b']))
+
+    @unittest.skip('TODO, subclasses do not complete yet')
+    def test_list_subclasses_complete(self):
+        class ListSubclass(list):
+            pass
+        self.assertSetEqual(self.com.matches(5, 'a[0].',
+                            locals_={'a': ListSubclass([Foo()])}),
+                            set(['method', 'a', 'b']))
+
+    def test_getitem_not_called_in_list_subclasses_overriding_getitem(self):
+        class FakeList(list):
+            def __getitem__(inner_self, i):
+                self.fail("possibly side-effecting __getitem_ method called")
+
+        self.com.matches(5, 'a[0].', locals_={'a': FakeList()})
+
+    def test_literals_complete(self):
+        self.assertSetEqual(self.com.matches(10, '[a][0][0].',
+                            locals_={'a': (Foo(),)}),
+                            set(['method', 'a', 'b']))
+
+    def test_dictionaries_complete(self):
+        self.assertSetEqual(self.com.matches(7, 'a["b"].',
+                            locals_={'a': {'b': Foo()}}),
+                            set(['method', 'a', 'b']))
+
 
 class TestMagicMethodCompletion(unittest.TestCase):
 
@@ -259,6 +391,13 @@ class TestMultilineJediCompletion(unittest.TestCase):
             [Comp('Abc', 'bc'), Comp('ade', 'de')])
         self.assertSetEqual(matches, set(['ade']))
 
+    @unittest.skipUnless(is_py34, 'asyncio required')
+    def test_issue_544(self):
+        com = autocomplete.MultilineJediCompletion()
+        code = '@asyncio.coroutine\ndef'
+        history = ('import asyncio', '@asyncio.coroutin')
+        com.matches(3, 'def', current_block=code, history=history)
+
 
 class TestGlobalCompletion(unittest.TestCase):
 
@@ -273,6 +412,23 @@ class TestGlobalCompletion(unittest.TestCase):
                                           locals_={'function': function}),
                          set(('function(', )))
 
+    def test_completions_are_unicode(self):
+        for m in self.com.matches(1, 'a', locals_={'abc': 10}):
+            self.assertIsInstance(m, type(u''))
+
+    @unittest.skipIf(py3, "in Python 3 invalid identifiers are passed through")
+    def test_ignores_nonascii_encodable(self):
+        self.assertEqual(self.com.matches(3, 'abc', locals_={'abcß': 10}),
+                         None)
+
+    def test_mock_kwlist(self):
+        with mock.patch.object(keyword, 'kwlist', new=['abcd']):
+            self.assertEqual(self.com.matches(3, 'abc', locals_={}), None)
+
+    def test_mock_kwlist_non_ascii(self):
+        with mock.patch.object(keyword, 'kwlist', new=['abcß']):
+            self.assertEqual(self.com.matches(3, 'abc', locals_={}), None)
+
 
 class TestParameterNameCompletion(unittest.TestCase):
     def test_set_of_params_returns_when_matches_found(self):
